@@ -15,6 +15,11 @@ if TYPE_CHECKING:
     import torch
 
 
+def generate_uuid() -> str:
+    from uuid import uuid1
+    return uuid1().hex
+
+
 def save_checkpoint():
     r"""
     Saves model checkpoints
@@ -31,6 +36,7 @@ def get_uuid():
 
 
 def create(*,
+           uuid: Optional[str] = None,
            name: Optional[str] = None,
            python_file: Optional[str] = None,
            comment: Optional[str] = None,
@@ -53,12 +59,16 @@ def create(*,
     """
 
     if writers is None:
-        writers = {'sqlite', 'tensorboard', 'web_api'}
+        writers = {'screen', 'sqlite', 'tensorboard', 'web_api'}
 
     if ignore_callers is None:
         ignore_callers = set()
 
-    _create_experiment(name=name,
+    if uuid is None:
+        uuid = generate_uuid()
+
+    _create_experiment(uuid=uuid,
+                       name=name,
                        python_file=python_file,
                        comment=comment,
                        writers=writers,
@@ -73,7 +83,8 @@ def evaluate():
     This will not record anything.
     """
 
-    _create_experiment(name=None,
+    _create_experiment(uuid=generate_uuid(),
+                       name=None,
                        python_file=None,
                        comment=None,
                        writers=set(),
@@ -82,11 +93,25 @@ def evaluate():
                        is_evaluate=True)
 
 
+def distributed(rank: int, world_size: int):
+    _experiment_singleton().distributed(rank, world_size)
+
+
 def add_model_savers(savers: Dict[str, ModelSaver]):
     _experiment_singleton().checkpoint_saver.add_savers(savers)
 
 
+@overload
+def add_pytorch_models(**kwargs: 'torch.nn.Module'):
+    ...
+
+
+@overload
 def add_pytorch_models(models: Dict[str, 'torch.nn.Module']):
+    ...
+
+
+def add_pytorch_models(*args, **kwargs):
     """
     Set variables for saving and loading
 
@@ -98,7 +123,13 @@ def add_pytorch_models(models: Dict[str, 'torch.nn.Module']):
     """
 
     from labml.internal.experiment.pytorch import add_models as _add_pytorch_models
-    _add_pytorch_models(models)
+
+    if len(args) > 0:
+        assert len(args) == 1
+        assert isinstance(args[0], dict)
+        _add_pytorch_models(args[0])
+    else:
+        _add_pytorch_models(kwargs)
 
 
 def add_sklearn_models(models: Dict[str, any]):
@@ -132,31 +163,11 @@ def configs(conf: BaseConfigs):
     ...
 
 
-@overload
-def configs(conf: BaseConfigs, run_order: List[Union[List[str], str]]):
-    ...
-
-
-@overload
-def configs(conf: BaseConfigs, *run_order: str):
-    ...
-
-
 @overload
 def configs(conf: BaseConfigs, conf_override: Dict[str, any]):
     ...
 
 
-@overload
-def configs(conf: BaseConfigs, conf_override: Dict[str, any], run_order: List[Union[List[str], str]]):
-    ...
-
-
-@overload
-def configs(conf: BaseConfigs, conf_override: Dict[str, any], *run_order: str):
-    ...
-
-
 def configs(*args):
     r"""
     Calculate configurations
@@ -192,42 +203,22 @@ def configs(*args):
         conf_dict (Dict[str, any], optional): a dictionary of configs
         conf_override (Dict[str, any], optional): a dictionary of
             configs to be overridden
-        run_order (List[Union[str, List[str]]], optional): list of
-            configs to be calculated and the order in which they should be
-            calculated. If not provided all configs will be calculated.
     """
-    configs_override: Optional[Dict[str, any]] = None
-    run_order: Optional[List[Union[List[str], str]]] = None
+    conf_override: Optional[Dict[str, any]] = None
+    conf = args[0]
     idx = 1
 
-    if isinstance(args[0], BaseConfigs):
-        if idx < len(args) and isinstance(args[idx], dict):
-            configs_override = args[idx]
-            idx += 1
+    if idx < len(args) and isinstance(args[idx], dict):
+        conf_override = args[idx]
+        idx += 1
 
-        if idx < len(args) and isinstance(args[idx], list):
-            run_order = args[idx]
-            if len(args) != idx + 1:
-                raise RuntimeError("Invalid call to calculate configs")
-            _experiment_singleton().calc_configs(args[0], configs_override, run_order)
-        else:
-            if idx == len(args):
-                _experiment_singleton().calc_configs(args[0], configs_override, run_order)
-            else:
-                run_order = list(args[idx:])
-                for key in run_order:
-                    if not isinstance(key, str):
-                        raise RuntimeError("Invalid call to calculate configs")
-                _experiment_singleton().calc_configs(args[0], configs_override, run_order)
-    elif isinstance(args[0], dict):
-        if idx < len(args) and isinstance(args[idx], dict):
-            configs_override = args[idx]
-            idx += 1
+    if len(args) != idx:
+        raise RuntimeError("Invalid call to calculate configs")
 
-        if idx != len(args):
-            raise RuntimeError("Invalid call to calculate configs")
-
-        _experiment_singleton().calc_configs_dict(args[0], configs_override)
+    if isinstance(conf, BaseConfigs):
+        _experiment_singleton().calc_configs(conf, conf_override)
+    elif isinstance(conf, dict):
+        _experiment_singleton().calc_configs(conf, conf_override)
     else:
         raise RuntimeError("Invalid call to calculate configs")
 
